@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 
+import pygtk
+pygtk.require('2.0')
 import gtk
 import atk
 
