@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -120,3 +120,23 @@ pygtk_target_list_to_list(GtkTargetList *targets)
     return list;
 }
 
+void
+pygtk_boxed_unref_shared(PyObject *boxed)
+{
+    if (boxed == Py_None) {
+        Py_DECREF(Py_None);
+        return;
+    }
+    PyGBoxed *pyboxed;
+    g_return_if_fail(boxed != NULL && PyObject_TypeCheck(boxed, &PyGBoxed_Type));
+    pyboxed = (PyGBoxed *) boxed;
+    if (pyboxed->ob_refcnt != 1) {
+        if (!pyboxed->free_on_dealloc) {
+            pyboxed->boxed = g_boxed_copy(pyboxed->gtype,
+                                          pyboxed->boxed);
+            pyboxed->free_on_dealloc = TRUE;
+        }
+    }
+    Py_DECREF(boxed);
+}
+
